#ifndef __VIDEO_LOCALIZATION__
#define __VIDEO_LOCALIZATION__

#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "DAG_SP_NODES_ONLY.h"


class VIDEO_LOCALIZATION : public Problem
{
public:
	VIDEO_LOCALIZATION() : N(0) {}
	~VIDEO_LOCALIZATION()
	{
		if (N == 0) return;
		int r;
		if (f) delete f;
		for (r=0; r<N; r++) delete P[r];
		delete [] P;
		delete [] start;
	}


	void set_time_ptr(double* _time_ptr) override
	{
		time_ptr = _time_ptr;
		if (N > 0)
		{
			int r;
			for (r=0; r<N; r++) P[r]->set_time_ptr(time_ptr);
		}
	}

	void InitShifts()
	{
		int r;
		node_num = edge_num = 0;
		for (r=0; r<N; r++)
		{
			node_num += P[r]->get_node_num();
			edge_num += P[r]->get_edge_num();
		}
		start = new int[N];
		int start_edge = 0;
		for (r=0; r<N; r++)
		{
			start[r] = (r == 0) ? 0 : start[r-1] + P[r-1]->get_node_num();
			if (r > 0) start_edge += P[r-1]->get_edge_num();

			P[r]->SetEdgeStart(node_num + start_edge - start[r]);
			P[r]->set_time_ptr(time_ptr);
		}
	}

	void Alloc(int _N, int* nn, int* mm)
	{
		int r;
		N = _N;
		P = new DAG_SP_NODES_ONLY*[N];
		node_num = 0;
		for (r=0; r<N; r++)
		{
			P[r] = new DAG_SP_NODES_ONLY(nn[r], mm[r]);
			node_num += nn[r];
		}
	}

	void Generate(unsigned long seed, int N1, int K1, int N2, int K2)
	{
		int nn[] = { N1*K1, N2*K2 };
		int mm[] = { (N1-1)*K1*K1+2*K1, (N2-1)*K2*K2+2*K2 };
		Alloc(2, nn, mm);

		P[0]->SetChain(N1, K1);
		P[1]->SetChain(N2, K2);
		InitShifts();

		srand(seed);
		f = new FunctionQ(node_num);
		f->SetRandom(seed);
	}
/*
	VIDEO_LOCALIZATION(int _node_num, int _edge_num_max) : node_num(_node_num), edge_num(0), edge_num_max(_edge_num_max), last_node(0)
	{
		nodes = new Node[node_num+2];
		edges = new Edge[edge_num_max];
		D = (new double[node_num+2])+1;
		back_ptr = (new int[node_num+2])+1;

		nodes[0].first_in = 0;

		time_ptr = NULL;
	}
	~VIDEO_LOCALIZATION()
	{
		if (f) delete f;
		delete [] nodes;
		delete [] edges;
		delete [] (D-1);
		delete [] (back_ptr-1);
	}

*/
	void Load()
	{
		N = 5;
		int nn[5] = { 160,      140,      140,      80,      140 };
		int mm[5] = { 2800+160, 2400+140, 2400+140, 1200+80, 2400+140 };

		Alloc(N, nn, mm);

		int i, j, r;

		f = new FunctionQ(node_num);
		
		////////
		FILE* fp = fopen("VIDEO_COLOCALIZATION_DATA/video_colocalization-Q.txt", "r");
		if (!fp) { printf("cannot open file\n"); exit(1); }
		for (i=0; i<node_num; i++)
		for (j=0; j<node_num; j++)
		{
			if (fscanf(fp, "%lf", &f->Q[i][j])!=1) { printf("Error 2 in Load\n"); exit(1); }
		}
//		if (fscanf(fp, "%lf", &x)!=0 && x!=0) { printf("Error 3 in Load\n"); exit(1); }
		fclose(fp);

		////////
		fp = fopen("VIDEO_COLOCALIZATION_DATA/video_colocalization-b.txt", "r");
		if (!fp) { printf("cannot open file\n"); exit(1); }
		for (i=0; i<node_num; i++)
		{
			if (fscanf(fp, "%lf", &f->b[i])!=1) { printf("Error 4 in Load\n"); exit(1); }
		}
//		if (fscanf(fp, "%lf", &x)!=0 && x!=0) { printf("Error 5 in Load\n"); exit(1); }
		fclose(fp);

		////////
		int idframes[660];
		int indegree[160];
		fp = fopen("VIDEO_COLOCALIZATION_DATA/video_colocalization-idframes.txt", "r");
		if (!fp) { printf("cannot open file\n"); exit(1); }
		for (i=0; i<node_num; i++)
		{
			if (fscanf(fp, "%d", &idframes[i])!=1) { printf("Error 6 in Load\n"); exit(1); }
		}
//		if (fscanf(fp, "%d", &i)!=0 && i!=0) { printf("Error 7 in Load\n"); exit(1); }
		fclose(fp);

		const char* edge_filenames[] = {
			"VIDEO_COLOCALIZATION_DATA/video_colocalization-edges0.txt",
			"VIDEO_COLOCALIZATION_DATA/video_colocalization-edges1.txt",
			"VIDEO_COLOCALIZATION_DATA/video_colocalization-edges2.txt",
			"VIDEO_COLOCALIZATION_DATA/video_colocalization-edges3.txt",
			"VIDEO_COLOCALIZATION_DATA/video_colocalization-edges4.txt"
		};
		int* IDFRAMES = idframes;
		int j_prev = -1;
		for (r=0; r<N; IDFRAMES += nn[r++])
		{
			int n = nn[r];
			memset(indegree, 0, n*sizeof(int));
			fp = fopen(edge_filenames[r], "r");
			if (!fp) { printf("cannot open file\n"); exit(1); }
			int jj = -1;
			while (fscanf(fp, "%d %d", &i, &j) == 2)
			{
				indegree[j] ++;
				while (jj < j)
				{
					jj ++;
					if (indegree[jj] == 0) P[r]->AddEdge(-1, jj);
				}
				P[r]->AddEdge(i, j);
			}
			while (jj < n-1)
			{
				jj ++;
				if (indegree[jj] == 0) P[r]->AddEdge(-1, jj);
			}
			fclose(fp);
			for (i=0; i<n; i++)
			{
				if (IDFRAMES[i] == IDFRAMES[n-1]) P[r]->AddEdge(i, n);
			}
			P[r]->FinalizeDAG();
		}

		InitShifts();
	}


	int get_n() override
	{
		return node_num+edge_num;
	} 

	int get_N() override
	{
		return node_num+edge_num;
	}

	void init_grad(Vector g) override
	{
		memset(g+node_num, 0, edge_num*sizeof(double));
	}
		
	void destroy_atom(Atom _a) override
	{
		int r;
		ATOM* a = (ATOM*)_a;
		for (r=0; r<N; r++) P[r]->destroy_atom(a->atoms[r]);
		delete a;
	}

	Atom get_init_atom()
	{
		int i;
		double* g = new double[node_num+edge_num];
		for (i=0; i<node_num+edge_num; i++) g[i] = (rand() % 100000);
		init_grad(g);
		Atom a = min_oracle(g, NO_TIME_LIMIT);
		delete [] g;
		return a;
	}

	void atom_to_extended_vector(Atom _a, ExtendedVector x) override
	{
		int r;
		ATOM* a = (ATOM*)_a;
		for (r=0; r<N; r++)
		{
			P[r]->atom_to_extended_vector(a->atoms[r], x+start[r]);
		}
	}

	void add_atom_to_extended_vector(double gamma, Atom _a, ExtendedVector x) override
	{
		int r;
		ATOM* a = (ATOM*)_a;
		for (r=0; r<N; r++) P[r]->add_atom_to_extended_vector(gamma, a->atoms[r], x+start[r]);
	}

	double dot_product(Atom _a, Vector x) override
	{
		double sum = 0;
		int r;
		ATOM* a = (ATOM*)_a;
		for (r=0; r<N; r++) sum += P[r]->dot_product(a->atoms[r], x+start[r]);
		return sum;
	}

	int compare_atoms(Atom _a, Atom _b) override
	{
		int r;
		ATOM* a = (ATOM*)_a;
		ATOM* b = (ATOM*)_b;
		for (r=0; r<N; r++)
		{
			int c = P[r]->compare_atoms(a->atoms[r], b->atoms[r]);
			if (c != 0) return c;
		}
		return 0;
	}

	double eval(ExtendedVector x) override
	{
		return f->eval(x);
	}

	void grad(ExtendedVector x, Vector g, double* fx) override
	{
		f->grad(x, g, fx);
	}

	double line_search(ExtendedVector x, ExtendedVector y, double* delta_f, double* delta_f1, double* delta_f_unconstrained) override
	{
		return f->line_search(x, y, delta_f, delta_f1, delta_f_unconstrained);
	}

	Atom min_oracle(Vector g, double time_limit) override
	{
		int r;
		ATOM* a = new ATOM(N);
		for (r=0; r<N; r++) a->atoms[r] = P[r]->min_oracle(g+start[r], time_limit);
		return a;
	}

	Atom minmax_oracle(Vector g, Vector x, Atom& _v, double time_limit) override
	{
		int r;
		ATOM* s = new ATOM(N);
		ATOM* v = new ATOM(N);
		_v = v;
		for (r=0; r<N; r++)
		{
			s->atoms[r] = P[r]->minmax_oracle(g+start[r], x+start[r], v->atoms[r], time_limit);
		}
		return s;
	}

	double move_DICG(Vector x, Atom _s, Atom _v) override
	{
		int r;
		ATOM* s = (ATOM*)_s;
		ATOM* v = (ATOM*)_v;

		double alpha = 1;
		for (r=0; r<N; r++)
		{
			double alpha_r = P[r]->get_alpha(x+start[r], v->atoms[r]);
			if (alpha > alpha_r) alpha = alpha_r;
		}

		for (r=0; r<N; r++)
		{
			P[r]->move_DICG(x+start[r], s->atoms[r], v->atoms[r], alpha);
		}

		return alpha;
	}

	Problem* reduce(Vector g, ExtendedVector x, Atom _s, ReducePolicy& policy, double time_limit) override
	{
		int i, r;
		ATOM* s = (ATOM*)_s;
		VIDEO_LOCALIZATION* Q = new VIDEO_LOCALIZATION();
		Q->N = N;
		Q->time_ptr = time_ptr;
		Q->P = new DAG_SP_NODES_ONLY*[N];
		int* node_map = (new int[node_num+2])+1;
		node_map[-1] = 0;
		for (r=0; r<N; r++)
		{
			P[r]->reduce_node_map = node_map + start[r];
			int tmp = P[r]->reduce_node_map[-1];
			Q->P[r] = (DAG_SP_NODES_ONLY*) P[r]->reduce(g+start[r], x+start[r], s->atoms[r], policy, time_limit);
			P[r]->reduce_node_map[-1] = tmp;
		}
		Q->InitShifts();
		for (r=0; r<N; r++)
		{
			int end = (r < N-1) ? start[r+1] : node_num;
			for (i=start[r]; i<end; i++)
			{
				if (node_map[i] >= 0) node_map[i] += Q->start[r];
			}
		}
		Q->f = f->shrink(Q->node_num, node_map);
		delete [] (node_map-1);
		return Q;
	}

	Atom shrink_atom(Problem* _B, Atom _a)
	{
		int r;
		VIDEO_LOCALIZATION* B = (VIDEO_LOCALIZATION*)_B;
		ATOM* a = (ATOM*)_a;
		ATOM* b = new ATOM(N);
		for (r=0; r<N; r++)
		{
			b->atoms[r] = P[r]->shrink_atom(B->P[r], a->atoms[r]);
			if (b->atoms[r] == NULL)
			{
				for (r--; r>=0; r--) P[r]->destroy_atom(b->atoms[r]);
				delete a;
				delete b;
				return NULL;
			}
		}
		delete a;
		return b;
	}
	Atom unshrink_atom(Problem* _A, Atom _a)
	{
		int r;
		VIDEO_LOCALIZATION* A = (VIDEO_LOCALIZATION*)_A;
		ATOM* a = (ATOM*)_a;
		ATOM* b = new ATOM(N);
		for (r=0; r<N; r++)
		{
			b->atoms[r] = P[r]->unshrink_atom(A->P[r], a->atoms[r]);
		}
		delete a;
		return b;
	}
	Atom reshape_atom(Problem* A, Problem* B, Atom a) override
	{
		if (A == this) A = NULL;
		if (B == this) B = NULL;
		if (A) a = unshrink_atom(A, a);
		if (B) a = shrink_atom(B, a);
		return a;
	}
	void reshape_x(Problem* _A, Problem* _B, ExtendedVector x) override
	{
		int r;
		if (_A == this) _A = NULL;
		if (_B == this) _B = NULL;
		if (_A)
		{
			VIDEO_LOCALIZATION* A = (VIDEO_LOCALIZATION*)_A;
			double* y = new double[A->node_num+A->edge_num];
			memcpy(y, x, (A->node_num+A->edge_num)*sizeof(double));
			for (r=0; r<N; r++)
			{
				DAG_SP_NODES_ONLY* PP = P[r];
				DAG_SP_NODES_ONLY* QQ = A->P[r];
				PP->unshrink_x(QQ, y+A->start[r], y+A->start[r]+QQ->GetEdgeStart(), x+start[r], x+start[r]+PP->GetEdgeStart());
			}
			delete [] y;
		}
		if (_B)
		{
			VIDEO_LOCALIZATION* B = (VIDEO_LOCALIZATION*)_B;
			double* y = new double[B->node_num+B->edge_num];
			for (r=0; r<N; r++)
			{
				DAG_SP_NODES_ONLY* PP = P[r];
				DAG_SP_NODES_ONLY* QQ = B->P[r];
				PP->shrink_x(QQ, x+start[r], x+start[r]+PP->GetEdgeStart(), y+B->start[r], y+B->start[r]+QQ->GetEdgeStart());
			}
			memcpy(x, y, (B->node_num+B->edge_num)*sizeof(double));
			delete [] y;
		}
	}


private:
	int N;
	int* start;
	int node_num;
	int edge_num;

	struct ATOM
	{
		ATOM(int N) { atoms = new Problem::Atom[N]; }
		~ATOM() { delete [] atoms; }
		Problem::Atom* atoms;
	};
	
	DAG_SP_NODES_ONLY** P;
	
	FunctionQ* f;
	

};


#endif
