import os, sys
from slugify import slugify
import sys, re, math
import matplotlib.pyplot as plt

ALG_BCG         = ('method=BCG  f_method=0', 'BCG','green', '-.')
ALG_DICG        = ('method=DICG  f_method=0', 'DiCG','blue', '-.')
ALG_DICGA       = ('method=DICG  f_method=0 DICG_method=AFW', 'DiCG$_A$','blue', '-.')
ALG_CacheDICG   = ('method=CacheDICG  f_method=0', 'CacheDiCG','red', '-.')
ALG_CacheDICGA  = ('method=CacheDICG  f_method=0 DICG_method=AFW', 'CacheDiCG$_A$','red', '-.')

ALG_BCG1         = ('method=BCG1  f_method=0', 'BCG1','green', '')
ALG_DICG1        = ('method=DICG1  f_method=0', 'DiCG1','blue', '')
ALG_DICGA1        = ('method=DICG1  f_method=0 DICG_method=AFW', 'DiCG$_A$1','blue', '')
ALG_CacheDICG1   = ('method=CacheDICG1  f_method=0', 'CacheDiCG1','red', '')
ALG_CacheDICGA1   = ('method=CacheDICG1  f_method=0 DICG_method=AFW', 'CacheDiCG$_A$1','red', '')

ALG_BCG2         = ('method=BCG2  f_method=0', 'BCG2','green', '--')
ALG_DICG2        = ('method=DICG2  f_method=0', 'DiCG2','blue', '--')
ALG_DICGA2        = ('method=DICG2  f_method=0 DICG_method=AFW', 'DiCG$_A$2','blue', '--')
ALG_CacheDICG2   = ('method=CacheDICG2  f_method=0', 'CacheDiCG2','red', '--')
ALG_CacheDICGA2   = ('method=CacheDICG2 f_method=0 DICG_method=AFW', 'CacheDiCG$_A$2','red', '--')

#ALG_DICGAX1        = ('method=DICG1  f_method=0 DICG_method=AFW DICG_eps=-2', 'DiCG$_A$1','yellow', '')
#ALG_DICGAX2        = ('method=DICG2  f_method=0 DICG_method=AFW DICG_eps=-1', 'DiCG$_A$2','yellow', '--')

######################################################################

def Run(argline):
	filename = 'LOGFILES/' + slugify(argline) + '.txt'
	if not os.path.isfile(filename):
#	if 1:
		print('running ' + argline)
		os.system('../run ' + argline + ' output=' + filename)
	fp = open(filename, 'r');
	s = fp.read()
	s = re.sub(r'\n====starting.*\n', ' ', s)
	s = s.split('\n')

	time_arr = []
	min_oracle_arr = []
	f_arr = []
	lower_bound_arr = []

	MIN_ORACLE = 0
	LOWER_BOUND = -1e300
	F = 1e300

	for line in s:
		depth = 1 if (len(line)>1 and line[0]=='*') else 0
		entries = re.split('\s+', line)
		L = {}
		for e in entries:
			x = e.split('=')
			if (len(x) == 2): L[x[0]] = x[1]
		if ('time' not in L): continue

		time = float(L['time'])
		if ('oracle' in L): MIN_ORACLE = int(L['oracle'])
		if ('f' in L): F = float(L['f'])
		if ('gap' in L and depth==0):
			gap = float(L['gap'])
			lower_bound = F-gap
			if (LOWER_BOUND < lower_bound): LOWER_BOUND = lower_bound

		time_arr.append(time)
		min_oracle_arr.append(MIN_ORACLE)
		lower_bound_arr.append(LOWER_BOUND)
		f_arr.append(F)

	return (time_arr, min_oracle_arr, lower_bound_arr, f_arr)


DELTA_V = {}

def compare_methods(name, problem, algs, time_max, eps, ybounds=None, xbound=None):
	name_base = name
	if (len(name)>2 and name[-2:]=='_R'): name_base = name[:-2]

	time_all = []
	min_oracle_all = []
	lower_bound_all = []
	f_all = []
	lower_bound_best = -1e300
	f_best = 1e300
	for alg in algs:
		(time_arr, min_oracle_arr, lower_bound_arr, f_arr) = Run(problem + ' ' + alg[0] + ' time_max=' + str(time_max))
		time_all.append(time_arr)
		min_oracle_all.append(min_oracle_arr)
		lower_bound_all.append(lower_bound_arr)
		f_all.append(f_arr)
		lower_bound_best = max(lower_bound_best, lower_bound_arr[-1])
		f_best = min(f_best, f_arr[-1])

	v = 0.5*(lower_bound_best + f_best)

	if (name_base in DELTA_V): v = DELTA_V[name_base]
	else:
		DELTA_V[name_base] = v
		print('delta_v = ' + str(v))

	for i in range(len(algs)):
		lower_bound_all[i] = [x-v for x in lower_bound_all[i]]
		f_all[i] = [x-v for x in f_all[i]]

	plt.figure(1)

	frame1 = plt.gca()
	frame1.axes.get_yaxis().set_ticks([])


	for i in range(len(algs)):
		if (name_base == 'simplex'): plt.plot(time_all[i], lower_bound_all[i], algs[i][3], color=algs[i][2], label=algs[i][1])
		else: plt.plot(time_all[i], lower_bound_all[i], algs[i][3], color=algs[i][2])
		plt.plot(time_all[i], f_all[i],           algs[i][3], color=algs[i][2])

	plt.legend(loc='upper right', prop={'size': 14})
	plt.xlabel('time')
	plt.yscale('symlog', linthreshy=eps)
	if (ybounds): plt.ylim(ybounds[0], ybounds[1])
	if (xbound): plt.xlim(0, xbound)

#	plt.figure(2)
#	for i in range(len(algs)):
#		plt.plot(min_oracle_all[i], lower_bound_all[i], algs[i][3], color=algs[i][2], label=algs[i][1])
#		plt.plot(min_oracle_all[i], f_all[i],           algs[i][3], color=algs[i][2])
#
#	plt.legend(loc='upper right')
#	plt.xlabel('oracle calls')
#	plt.yscale('symlog', linthreshy=eps)
#	if (ybounds): plt.ylim(ybounds[0], ybounds[1])

#	plt.show()

	plt.savefig(name + '.eps', format='eps')
	plt.clf()

######################################################################################################

run_simplex         = 1
run_dag_sp          = 1
run_pm_bipartite_10 = 1
run_pm_bipartite_100 = 1
run_birkhoff        = 1
run_video           = 1
run_pm_e10          = 1
run_pm_e100         = 1
run_pm_r10          = 1
run_pm_r100         = 1


if run_simplex:
	problem = '"SIMPLEX(n=1000,m=100,sigma=0.1,skewed=1,seed=1)"'
	compare_methods('simplex_R', problem, [ALG_BCG1, ALG_DICG1, ALG_CacheDICG1, ALG_BCG2, ALG_DICG2, ALG_CacheDICG2], 6, 1e-3, (-1e2, 1e2))
	compare_methods('simplex',   problem, [ALG_BCG, ALG_DICG, ALG_CacheDICG], 60, 1e-3, (-1e2, 1e2))

if run_dag_sp:
	problem = '"DAG_SP(N=100,K=10,m=100,sigma=19,seed=1)"'
	compare_methods('dag_sp_R', problem, [ALG_BCG1, ALG_DICG1, ALG_CacheDICG1, ALG_BCG2, ALG_DICG2, ALG_CacheDICG2], 40, 1e-3, (-1e3, 1e3))
	compare_methods('dag_sp', problem, [ALG_BCG, ALG_DICG, ALG_CacheDICG], 400, 1e-3, (-1e3, 1e3))

if run_pm_bipartite_10:
	problem = '"BIRKHOFF(N=500,degree=10,m=100,sigma=63,seed=1)"'
	compare_methods('pm_bipartite_10_R', problem, [ALG_BCG1,  ALG_DICGA1, ALG_CacheDICGA1, ALG_BCG2, ALG_DICGA2, ALG_CacheDICGA2], 200, 1e-3, (-1e1, 1e1))
	compare_methods('pm_bipartite_10',   problem, [ALG_BCG, ALG_DICGA, ALG_CacheDICGA], 2000, 1e-3, (-1e1, 1e1))

if run_pm_bipartite_100:
	problem = '"BIRKHOFF(N=500,degree=100,m=100,sigma=108,seed=1)"'
	compare_methods('pm_bipartite_100_R', problem, [ALG_BCG1,  ALG_DICGA1, ALG_CacheDICGA1, ALG_BCG2, ALG_DICGA2, ALG_CacheDICGA2], 200, 1e-3, (-1e4, 1e4))
	compare_methods('pm_bipartite_100',   problem, [ALG_BCG, ALG_DICGA, ALG_CacheDICGA], 2000, 1e-3, (-1e4, 1e4))

if run_birkhoff:
	problem = '"BIRKHOFF(N=500,m=100,sigma=150,seed=1)"'
	compare_methods('birkhoff_R', problem, [ALG_BCG1,  ALG_DICGA1, ALG_CacheDICGA1, ALG_BCG2, ALG_DICGA2, ALG_CacheDICGA2], 300, 1e-3, (-1e5, 1e5))
	compare_methods('birkhoff',   problem, [ALG_BCG, ALG_DICGA, ALG_CacheDICGA], 2000, 1e-3, (-1e5, 1e5))

if run_video:
	problem = '"VIDEO_LOCALIZATION()"'
	compare_methods('video_R', problem, [ALG_BCG1, ALG_DICG1, ALG_CacheDICG1, ALG_BCG2, ALG_DICG2, ALG_CacheDICG2], 0.1, 1e-6, (-1e-1, 1e-1))
	compare_methods('video',   problem, [ALG_BCG, ALG_DICG, ALG_CacheDICG], 2, 1e-6, (-1e-1, 1e-1))

if run_pm_r10:
	problem = '"PM(file=./blossom5-instances/Boost_1000_10_1.g,m=100,sigma=96,seed=1)"'
	compare_methods('pm_r10_R', problem, [ALG_BCG1, ALG_BCG2], 200, 1e-3, (-1e4, 1e4))
	compare_methods('pm_r10', problem, [ALG_BCG], 2000, 1e-3, (-1e4, 1e4))

if run_pm_r100:
	problem = '"PM(file=./blossom5-instances/Boost_1000_100_1.g,m=100,sigma=136,seed=1)"'
	compare_methods('pm_r100_R', problem, [ALG_BCG1, ALG_BCG2], 200, 1e-3, (-1e4, 1e4))
	compare_methods('pm_r100', problem, [ALG_BCG], 2000, 1e-3, (-1e4, 1e4))

if run_pm_e10:
	problem = '"PM(file=./blossom5-instances/Euclid2d_1000_10_1.g,m=100,sigma=94,seed=1)"'
	compare_methods('pm_e10_R', problem, [ALG_BCG1, ALG_BCG2], 200, 1e-3, (-1e4, 1e4))
	compare_methods('pm_e10',   problem, [ALG_BCG], 2000, 1e-3, (-1e4, 1e4))

if run_pm_e100:
	problem = '"PM(file=./blossom5-instances/Euclid2d_1000_100_1.g,m=200,sigma=122,seed=1)"'
	compare_methods('pm_e100_R', problem, [ALG_BCG1, ALG_BCG2], 200, 1e-3, (-1e4, 1e4))
	compare_methods('pm_e100', problem, [ALG_BCG], 2000, 1e-3, (-1e4, 1e4))



# birkohff_small_R DICG1 DICG2
